{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "1qcdnuAet_El"
   },
   "source": [
    "# Prerequisites\n",
    "Run this cell to prepare the environment. This step is obligatory."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/"
    },
    "executionInfo": {
     "elapsed": 25578,
     "status": "ok",
     "timestamp": 1683188242647,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "bieK6_UAZ94I",
    "outputId": "9979688d-1f14-4a2f-d733-c261b3cce373",
    "pycharm": {
     "is_executing": true
    }
   },
   "outputs": [],
   "source": [
    "!pip install retentioneering"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "rwHSi8hf1jlc"
   },
   "source": [
    "The full text of [TransitionGraph](https://doc.retentioneering.com/release3/doc/user_guides/transition_graph.html) user guide is available on the retentioneering website."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# TransitionGraph"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "B_l0s_3Ax_Dx"
   },
   "source": [
    "## Loading data"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {
    "executionInfo": {
     "elapsed": 7,
     "status": "ok",
     "timestamp": 1683188260267,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "FJdD7NNXl_lV"
   },
   "outputs": [],
   "source": [
    "import retentioneering\n",
    "import pandas as pd\n",
    "\n",
    "from retentioneering import datasets\n",
    "\n",
    "stream = datasets.load_simple_shop()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "DRYrsb_08DIY"
   },
   "source": [
    "## A basic example"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/",
     "height": 0
    },
    "executionInfo": {
     "elapsed": 392,
     "status": "ok",
     "timestamp": 1683188260654,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "3wllHdNvZJJV",
    "outputId": "066f9a72-23a7-4437-ebe0-78cb4fcbcdfa"
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"el5Z8IYG\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('el5Z8IYG')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('el5Z8IYG')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('el5Z8IYG')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('el5Z8IYG').contentDocument\n",
       "      const iframeWindow = document.getElementById('el5Z8IYG').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elPATRXO`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '65d47e15-9361-449f-954b-8a845b9b905c',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16751618680259, \"source\": 565}, \"user_id\": {\"degree\": 8.32844087510385, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.488221518115443, \"source\": 2172}, \"user_id\": {\"degree\": 15.880365549709222, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 2, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.458878633420581, \"source\": 706}, \"user_id\": {\"degree\": 9.425090002769316, \"source\": 653}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 3, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 34.0, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 4, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.483950957432153, \"source\": 1686}, \"user_id\": {\"degree\": 15.265577402381613, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 5, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.13059650089544, \"source\": 1515}, \"user_id\": {\"degree\": 13.321517585156467, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 6, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72337787574046, \"source\": 834}, \"user_id\": {\"degree\": 10.214345056770977, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 7, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.407080865132938, \"source\": 197}, \"user_id\": {\"degree\": 5.578510108003323, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 8, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.045598567295771, \"source\": 506}, \"user_id\": {\"degree\": 7.89642758238715, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 9, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.287505166000827, \"source\": 1107}, \"user_id\": {\"degree\": 11.959014123511494, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 10, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.644165863066538, \"source\": 5635}, \"user_id\": {\"degree\": 23.814455829410136, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 11, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.872709739633558, \"source\": 2842}, \"user_id\": {\"degree\": 19.98449183051786, \"source\": 1924}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}],\n",
       "    configLinks: [{\"sourceIndex\": 11, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.00020588840848260242, \"weight\": 1}, \"user_id\": {\"weight_norm\": 0.0004962779156327543, \"weight\": 1}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11756228124356599, \"weight\": 571}, \"user_id\": {\"weight_norm\": 0.23722084367245658, \"weight\": 478}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.3471278567016677, \"weight\": 1686}, \"user_id\": {\"weight_norm\": 0.6729528535980149, \"weight\": 1356}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.047971999176446366, \"weight\": 233}, \"user_id\": {\"weight_norm\": 0.10124069478908189, \"weight\": 204}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.35186329009676753, \"weight\": 1709}, \"user_id\": {\"weight_norm\": 0.6570719602977667, \"weight\": 1324}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 4857}, \"user_id\": {\"weight_norm\": 0.9945409429280397, \"weight\": 2004}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.5239859995882232, \"weight\": 2545}, \"user_id\": {\"weight_norm\": 0.7344913151364765, \"weight\": 1480}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.3119209388511427, \"weight\": 1515}, \"user_id\": {\"weight_norm\": 0.5568238213399503, \"weight\": 1122}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.44718962322421246, \"weight\": 2172}, \"user_id\": {\"weight_norm\": 0.7096774193548387, \"weight\": 1430}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.03664813670990323, \"weight\": 178}, \"user_id\": {\"weight_norm\": 0.08535980148883375, \"weight\": 172}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.17171093267449042, \"weight\": 834}, \"user_id\": {\"weight_norm\": 0.37121588089330027, \"weight\": 748}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.10417953469219683, \"weight\": 506}, \"user_id\": {\"weight_norm\": 0.2327543424317618, \"weight\": 469}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.014618077002264772, \"weight\": 71}, \"user_id\": {\"weight_norm\": 0.0337468982630273, \"weight\": 68}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.007000205888408483, \"weight\": 34}, \"user_id\": {\"weight_norm\": 0.01687344913151365, \"weight\": 34}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.15482808317891703, \"weight\": 752}, \"user_id\": {\"weight_norm\": 0.3389578163771712, \"weight\": 683}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.011529750875025736, \"weight\": 56}, \"user_id\": {\"weight_norm\": 0.02729528535980149, \"weight\": 55}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.07309038501132387, \"weight\": 355}, \"user_id\": {\"weight_norm\": 0.16476426799007443, \"weight\": 332}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.8367304920732963, \"weight\": 4064}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 2015}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.21453572163887172, \"weight\": 1042}, \"user_id\": {\"weight_norm\": 0.2992555831265509, \"weight\": 603}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.0041177681696520484, \"weight\": 20}, \"user_id\": {\"weight_norm\": 0.009925558312655087, \"weight\": 20}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.10438542310067943, \"weight\": 507}, \"user_id\": {\"weight_norm\": 0.23573200992555832, \"weight\": 475}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.009676755198682314, \"weight\": 47}, \"user_id\": {\"weight_norm\": 0.022332506203473945, \"weight\": 45}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.021412394482190653, \"weight\": 104}, \"user_id\": {\"weight_norm\": 0.050620347394540945, \"weight\": 102}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.02347127856701668, \"weight\": 114}, \"user_id\": {\"weight_norm\": 0.05359801488833747, \"weight\": 108}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.008647313156269302, \"weight\": 42}, \"user_id\": {\"weight_norm\": 0.020347394540942927, \"weight\": 41}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.11632695079267037, \"weight\": 565}, \"user_id\": {\"weight_norm\": 0.25856079404466503, \"weight\": 521}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.04056001647107268, \"weight\": 197}, \"user_id\": {\"weight_norm\": 0.09429280397022333, \"weight\": 190}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.01955939880584723, \"weight\": 95}, \"user_id\": {\"weight_norm\": 0.04665012406947891, \"weight\": 94}}, \"type\": \"suit\"}, {\"sourceIndex\": 2, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.05538398188182005, \"weight\": 269}, \"user_id\": {\"weight_norm\": 0.12009925558312655, \"weight\": 242}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.09717932880378835, \"weight\": 472}, \"user_id\": {\"weight_norm\": 0.2138957816377171, \"weight\": 431}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.15441630636195183, \"weight\": 750}, \"user_id\": {\"weight_norm\": 0.3076923076923077, \"weight\": 620}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.02470660901791229, \"weight\": 120}, \"user_id\": {\"weight_norm\": 0.05657568238213399, \"weight\": 114}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.13588634959851761, \"weight\": 660}, \"user_id\": {\"weight_norm\": 0.288833746898263, \"weight\": 582}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.26724315421041794, \"weight\": 1298}, \"user_id\": {\"weight_norm\": 0.4635235732009926, \"weight\": 934}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.018735845171916822, \"weight\": 91}, \"user_id\": {\"weight_norm\": 0.04367245657568238, \"weight\": 88}}, \"type\": \"suit\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    linksThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'event_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'event_id',\n",
       "    showWeights: true,\n",
       "    showPercents: false,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: true,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(0),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'el5Z8IYG_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('el5Z8IYG')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"el5Z8IYG_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"el36MU80\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('el36MU80')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('el36MU80')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('el36MU80')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('el36MU80').contentDocument\n",
       "      const iframeWindow = document.getElementById('el36MU80').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elPATRXO`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '65d47e15-9361-449f-954b-8a845b9b905c',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'el36MU80_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('el36MU80')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"el36MU80_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "stream.transition_graph();"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Transition graph parameters"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "1uPdv0rcGqMq"
   },
   "source": [
    "#### Setting the weight options"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/",
     "height": 0
    },
    "executionInfo": {
     "elapsed": 8,
     "status": "ok",
     "timestamp": 1683188260654,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "x1TI8L2EHQO0",
    "outputId": "fc3e41d5-471a-4bb2-f0c4-e208bb3d157d"
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"elKSDF47\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elKSDF47')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elKSDF47')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elKSDF47')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elKSDF47').contentDocument\n",
       "      const iframeWindow = document.getElementById('elKSDF47').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=el5SF2XS`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '99ad5dd1-42e3-4856-b68e-d66b7f768745',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16752, \"source\": 565}, \"user_id\": {\"degree\": 8.32844, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.48822, \"source\": 2172}, \"user_id\": {\"degree\": 15.88037, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 2, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.45888, \"source\": 706}, \"user_id\": {\"degree\": 9.42509, \"source\": 653}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 3, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 34.0, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 4, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.48395, \"source\": 1686}, \"user_id\": {\"degree\": 15.26558, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 5, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.1306, \"source\": 1515}, \"user_id\": {\"degree\": 13.32152, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 6, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72338, \"source\": 834}, \"user_id\": {\"degree\": 10.21435, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 7, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.40708, \"source\": 197}, \"user_id\": {\"degree\": 5.57851, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 8, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.0456, \"source\": 506}, \"user_id\": {\"degree\": 7.89643, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 9, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.28751, \"source\": 1107}, \"user_id\": {\"degree\": 11.95901, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 10, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.64417, \"source\": 5635}, \"user_id\": {\"degree\": 23.81446, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 11, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.87271, \"source\": 2842}, \"user_id\": {\"degree\": 19.98449, \"source\": 1924}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}],\n",
       "    configLinks: [{\"sourceIndex\": 11, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.0004, \"weight\": 0.0004}, \"user_id\": {\"weight_norm\": 0.00059, \"weight\": 0.00059}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.22923, \"weight\": 0.22923}, \"user_id\": {\"weight_norm\": 0.28385, \"weight\": 0.28385}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.67684, \"weight\": 0.67684}, \"user_id\": {\"weight_norm\": 0.80523, \"weight\": 0.80523}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.09354, \"weight\": 0.09354}, \"user_id\": {\"weight_norm\": 0.12114, \"weight\": 0.12114}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.13354, \"weight\": 0.13354}, \"user_id\": {\"weight_norm\": 0.41846, \"weight\": 0.41846}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.37951, \"weight\": 0.37951}, \"user_id\": {\"weight_norm\": 0.63338, \"weight\": 0.63338}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.19886, \"weight\": 0.19886}, \"user_id\": {\"weight_norm\": 0.46776, \"weight\": 0.46776}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.11838, \"weight\": 0.11838}, \"user_id\": {\"weight_norm\": 0.35461, \"weight\": 0.35461}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.16971, \"weight\": 0.16971}, \"user_id\": {\"weight_norm\": 0.45196, \"weight\": 0.45196}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11202, \"weight\": 0.11202}, \"user_id\": {\"weight_norm\": 0.13501, \"weight\": 0.13501}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.52486, \"weight\": 0.52486}, \"user_id\": {\"weight_norm\": 0.58713, \"weight\": 0.58713}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.31844, \"weight\": 0.31844}, \"user_id\": {\"weight_norm\": 0.36813, \"weight\": 0.36813}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04468, \"weight\": 0.04468}, \"user_id\": {\"weight_norm\": 0.05338, \"weight\": 0.05338}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04326, \"weight\": 0.04326}, \"user_id\": {\"weight_norm\": 0.04809, \"weight\": 0.04809}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.95674, \"weight\": 0.95674}, \"user_id\": {\"weight_norm\": 0.96605, \"weight\": 0.96605}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.13625, \"weight\": 0.13625}, \"user_id\": {\"weight_norm\": 0.1455, \"weight\": 0.1455}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.86375, \"weight\": 0.86375}, \"user_id\": {\"weight_norm\": 0.87831, \"weight\": 0.87831}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.79593, \"weight\": 0.79593}, \"user_id\": {\"weight_norm\": 0.94601, \"weight\": 0.94601}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.20407, \"weight\": 0.20407}, \"user_id\": {\"weight_norm\": 0.2831, \"weight\": 0.2831}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.03795, \"weight\": 0.03795}, \"user_id\": {\"weight_norm\": 0.04098, \"weight\": 0.04098}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.96205, \"weight\": 0.96205}, \"user_id\": {\"weight_norm\": 0.97336, \"weight\": 0.97336}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.31126, \"weight\": 0.31126}, \"user_id\": {\"weight_norm\": 0.3125, \"weight\": 0.3125}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.68874, \"weight\": 0.68874}, \"user_id\": {\"weight_norm\": 0.70833, \"weight\": 0.70833}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11254, \"weight\": 0.11254}, \"user_id\": {\"weight_norm\": 0.12343, \"weight\": 0.12343}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04146, \"weight\": 0.04146}, \"user_id\": {\"weight_norm\": 0.04686, \"weight\": 0.04686}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.55775, \"weight\": 0.55775}, \"user_id\": {\"weight_norm\": 0.59543, \"weight\": 0.59543}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.19447, \"weight\": 0.19447}, \"user_id\": {\"weight_norm\": 0.21714, \"weight\": 0.21714}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.09378, \"weight\": 0.09378}, \"user_id\": {\"weight_norm\": 0.10743, \"weight\": 0.10743}}, \"type\": \"suit\"}, {\"sourceIndex\": 2, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.35171, \"weight\": 0.35171}, \"user_id\": {\"weight_norm\": 0.4336, \"weight\": 0.4336}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.55887, \"weight\": 0.55887}, \"user_id\": {\"weight_norm\": 0.62374, \"weight\": 0.62374}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.08942, \"weight\": 0.08942}, \"user_id\": {\"weight_norm\": 0.11469, \"weight\": 0.11469}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.32211, \"weight\": 0.32211}, \"user_id\": {\"weight_norm\": 0.43304, \"weight\": 0.43304}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.63348, \"weight\": 0.63348}, \"user_id\": {\"weight_norm\": 0.69494, \"weight\": 0.69494}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04441, \"weight\": 0.04441}, \"user_id\": {\"weight_norm\": 0.06548, \"weight\": 0.06548}}, \"type\": \"suit\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    linksThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'user_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'user_id',\n",
       "    showWeights: true,\n",
       "    showPercents: false,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: true,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(0),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elKSDF47_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elKSDF47')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elKSDF47_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"elEZRS0F\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elEZRS0F')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elEZRS0F')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elEZRS0F')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elEZRS0F').contentDocument\n",
       "      const iframeWindow = document.getElementById('elEZRS0F').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=el5SF2XS`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '99ad5dd1-42e3-4856-b68e-d66b7f768745',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elEZRS0F_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elEZRS0F')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elEZRS0F_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "stream.transition_graph(\n",
    "    edges_norm_type='node',\n",
    "    edges_weight_col='user_id'\n",
    ");"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "RwMnbDHMHozf"
   },
   "source": [
    "### Thresholds"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/",
     "height": 0
    },
    "executionInfo": {
     "elapsed": 311,
     "status": "ok",
     "timestamp": 1683188260959,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "5wGC_mp1H_OH",
    "outputId": "6d4b8d21-91bc-4974-c440-1b71795dd8ee"
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"elSG9BRX\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elSG9BRX')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elSG9BRX')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elSG9BRX')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elSG9BRX').contentDocument\n",
       "      const iframeWindow = document.getElementById('elSG9BRX').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elBVYC35`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '636df1b1-4b4c-4850-8c63-7778e50f4fae',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16752, \"source\": 565}, \"user_id\": {\"degree\": 8.32844, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.48822, \"source\": 2172}, \"user_id\": {\"degree\": 15.88037, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 2, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.45888, \"source\": 706}, \"user_id\": {\"degree\": 9.42509, \"source\": 653}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 3, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 34.0, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 4, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.48395, \"source\": 1686}, \"user_id\": {\"degree\": 15.26558, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 5, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.1306, \"source\": 1515}, \"user_id\": {\"degree\": 13.32152, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 6, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72338, \"source\": 834}, \"user_id\": {\"degree\": 10.21435, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 7, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.40708, \"source\": 197}, \"user_id\": {\"degree\": 5.57851, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 8, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.0456, \"source\": 506}, \"user_id\": {\"degree\": 7.89643, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 9, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.28751, \"source\": 1107}, \"user_id\": {\"degree\": 11.95901, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 10, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.64417, \"source\": 5635}, \"user_id\": {\"degree\": 23.81446, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 11, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.87271, \"source\": 2842}, \"user_id\": {\"degree\": 19.98449, \"source\": 1924}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}],\n",
       "    configLinks: [{\"sourceIndex\": 11, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.0004, \"weight\": 0.0004}, \"user_id\": {\"weight_norm\": 0.00059, \"weight\": 0.00059}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.22923, \"weight\": 0.22923}, \"user_id\": {\"weight_norm\": 0.28385, \"weight\": 0.28385}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.67684, \"weight\": 0.67684}, \"user_id\": {\"weight_norm\": 0.80523, \"weight\": 0.80523}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.09354, \"weight\": 0.09354}, \"user_id\": {\"weight_norm\": 0.12114, \"weight\": 0.12114}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.13354, \"weight\": 0.13354}, \"user_id\": {\"weight_norm\": 0.41846, \"weight\": 0.41846}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.37951, \"weight\": 0.37951}, \"user_id\": {\"weight_norm\": 0.63338, \"weight\": 0.63338}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.19886, \"weight\": 0.19886}, \"user_id\": {\"weight_norm\": 0.46776, \"weight\": 0.46776}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.11838, \"weight\": 0.11838}, \"user_id\": {\"weight_norm\": 0.35461, \"weight\": 0.35461}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.16971, \"weight\": 0.16971}, \"user_id\": {\"weight_norm\": 0.45196, \"weight\": 0.45196}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11202, \"weight\": 0.11202}, \"user_id\": {\"weight_norm\": 0.13501, \"weight\": 0.13501}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.52486, \"weight\": 0.52486}, \"user_id\": {\"weight_norm\": 0.58713, \"weight\": 0.58713}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.31844, \"weight\": 0.31844}, \"user_id\": {\"weight_norm\": 0.36813, \"weight\": 0.36813}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04468, \"weight\": 0.04468}, \"user_id\": {\"weight_norm\": 0.05338, \"weight\": 0.05338}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04326, \"weight\": 0.04326}, \"user_id\": {\"weight_norm\": 0.04809, \"weight\": 0.04809}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.95674, \"weight\": 0.95674}, \"user_id\": {\"weight_norm\": 0.96605, \"weight\": 0.96605}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.13625, \"weight\": 0.13625}, \"user_id\": {\"weight_norm\": 0.1455, \"weight\": 0.1455}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.86375, \"weight\": 0.86375}, \"user_id\": {\"weight_norm\": 0.87831, \"weight\": 0.87831}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.79593, \"weight\": 0.79593}, \"user_id\": {\"weight_norm\": 0.94601, \"weight\": 0.94601}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.20407, \"weight\": 0.20407}, \"user_id\": {\"weight_norm\": 0.2831, \"weight\": 0.2831}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.03795, \"weight\": 0.03795}, \"user_id\": {\"weight_norm\": 0.04098, \"weight\": 0.04098}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.96205, \"weight\": 0.96205}, \"user_id\": {\"weight_norm\": 0.97336, \"weight\": 0.97336}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.31126, \"weight\": 0.31126}, \"user_id\": {\"weight_norm\": 0.3125, \"weight\": 0.3125}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.68874, \"weight\": 0.68874}, \"user_id\": {\"weight_norm\": 0.70833, \"weight\": 0.70833}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11254, \"weight\": 0.11254}, \"user_id\": {\"weight_norm\": 0.12343, \"weight\": 0.12343}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04146, \"weight\": 0.04146}, \"user_id\": {\"weight_norm\": 0.04686, \"weight\": 0.04686}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.55775, \"weight\": 0.55775}, \"user_id\": {\"weight_norm\": 0.59543, \"weight\": 0.59543}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.19447, \"weight\": 0.19447}, \"user_id\": {\"weight_norm\": 0.21714, \"weight\": 0.21714}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.09378, \"weight\": 0.09378}, \"user_id\": {\"weight_norm\": 0.10743, \"weight\": 0.10743}}, \"type\": \"suit\"}, {\"sourceIndex\": 2, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.35171, \"weight\": 0.35171}, \"user_id\": {\"weight_norm\": 0.4336, \"weight\": 0.4336}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.55887, \"weight\": 0.55887}, \"user_id\": {\"weight_norm\": 0.62374, \"weight\": 0.62374}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.08942, \"weight\": 0.08942}, \"user_id\": {\"weight_norm\": 0.11469, \"weight\": 0.11469}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.32211, \"weight\": 0.32211}, \"user_id\": {\"weight_norm\": 0.43304, \"weight\": 0.43304}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.63348, \"weight\": 0.63348}, \"user_id\": {\"weight_norm\": 0.69494, \"weight\": 0.69494}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04441, \"weight\": 0.04441}, \"user_id\": {\"weight_norm\": 0.06548, \"weight\": 0.06548}}, \"type\": \"suit\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"event_id\": 500},\n",
       "    linksThreshold: {\"user_id\": 0.12},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'user_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'user_id',\n",
       "    showWeights: true,\n",
       "    showPercents: false,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: true,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(0),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elSG9BRX_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elSG9BRX')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elSG9BRX_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"elY66M50\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elY66M50')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elY66M50')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elY66M50')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elY66M50').contentDocument\n",
       "      const iframeWindow = document.getElementById('elY66M50').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elBVYC35`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '636df1b1-4b4c-4850-8c63-7778e50f4fae',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elY66M50_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elY66M50')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elY66M50_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "stream.transition_graph(\n",
    "    edges_norm_type='node',\n",
    "    edges_weight_col='user_id',\n",
    "    edges_threshold={'user_id': 0.12},\n",
    "    nodes_threshold={'event_id': 500}\n",
    ");"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "qCpWRTFVIN5j"
   },
   "source": [
    "### Targets"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/",
     "height": 0
    },
    "executionInfo": {
     "elapsed": 1243,
     "status": "ok",
     "timestamp": 1683188262200,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "qVfwLcY1I0Ov",
    "outputId": "a8c9e404-e819-442b-d16b-d92a02e03a14",
    "scrolled": false
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"elMEQHO5\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elMEQHO5')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elMEQHO5')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elMEQHO5')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elMEQHO5').contentDocument\n",
       "      const iframeWindow = document.getElementById('elMEQHO5').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elTR4N7H`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '442b958c-1825-4d69-a2b7-a5cd33c64d08',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16751618680259, \"source\": 565}, \"user_id\": {\"degree\": 8.166888829645428, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.488221518115443, \"source\": 2172}, \"user_id\": {\"degree\": 15.436950146627566, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 2, \"name\": \"path_start\", \"degree\": {\"event_id\": {\"degree\": 11.751067640170822, \"source\": 3751}, \"user_id\": {\"degree\": 34.0, \"source\": 3751}}, \"type\": \"source_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 3, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.458878633420581, \"source\": 706}, \"user_id\": {\"degree\": 9.222607304718743, \"source\": 653}}, \"type\": \"nice_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 4, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 32.880298587043455, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 5, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.483950957432153, \"source\": 1686}, \"user_id\": {\"degree\": 14.845107971207678, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 6, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.13059650089544, \"source\": 1515}, \"user_id\": {\"degree\": 12.973607038123168, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 7, \"name\": \"path_end\", \"degree\": {\"event_id\": {\"degree\": 11.751067640170822, \"source\": 3751}, \"user_id\": {\"degree\": 34.0, \"source\": 3751}}, \"type\": \"bad_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 8, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72337787574046, \"source\": 834}, \"user_id\": {\"degree\": 9.982404692082111, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 9, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.407080865132938, \"source\": 197}, \"user_id\": {\"degree\": 5.519594774726739, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 10, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.045598567295771, \"source\": 506}, \"user_id\": {\"degree\": 7.750999733404425, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 11, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.287505166000827, \"source\": 1107}, \"user_id\": {\"degree\": 11.66195681151693, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 12, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.644165863066538, \"source\": 5635}, \"user_id\": {\"degree\": 23.074913356438284, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 13, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.872709739633558, \"source\": 2842}, \"user_id\": {\"degree\": 19.38789656091709, \"source\": 1924}}, \"type\": \"nice_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}],\n",
       "    configLinks: [{\"sourceIndex\": 13, \"targetIndex\": 13, \"weights\": {\"event_id\": {\"weight_norm\": 0.00020588840848260242, \"weight\": 1}, \"user_id\": {\"weight_norm\": 0.00037230081906180194, \"weight\": 1}}, \"type\": \"nice\"}, {\"sourceIndex\": 13, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.11756228124356599, \"weight\": 571}, \"user_id\": {\"weight_norm\": 0.17795979151154132, \"weight\": 478}}, \"type\": \"suit\"}, {\"sourceIndex\": 13, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.3471278567016677, \"weight\": 1686}, \"user_id\": {\"weight_norm\": 0.5048399106478034, \"weight\": 1356}}, \"type\": \"suit\"}, {\"sourceIndex\": 13, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.047971999176446366, \"weight\": 233}, \"user_id\": {\"weight_norm\": 0.0759493670886076, \"weight\": 204}}, \"type\": \"suit\"}, {\"sourceIndex\": 13, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.07226683137739345, \"weight\": 351}, \"user_id\": {\"weight_norm\": 0.13067758749069247, \"weight\": 351}}, \"type\": \"bad\"}, {\"sourceIndex\": 4, \"targetIndex\": 13, \"weights\": {\"event_id\": {\"weight_norm\": 0.35186329009676753, \"weight\": 1709}, \"user_id\": {\"weight_norm\": 0.49292628443782577, \"weight\": 1324}}, \"type\": \"nice\"}, {\"sourceIndex\": 4, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 4857}, \"user_id\": {\"weight_norm\": 0.746090841399851, \"weight\": 2004}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.5239859995882232, \"weight\": 2545}, \"user_id\": {\"weight_norm\": 0.5510052122114668, \"weight\": 1480}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.3541280625900762, \"weight\": 1720}, \"user_id\": {\"weight_norm\": 0.6403574087862993, \"weight\": 1720}}, \"type\": \"bad\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.3119209388511427, \"weight\": 1515}, \"user_id\": {\"weight_norm\": 0.4177215189873418, \"weight\": 1122}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.44718962322421246, \"weight\": 2172}, \"user_id\": {\"weight_norm\": 0.5323901712583767, \"weight\": 1430}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.03664813670990323, \"weight\": 178}, \"user_id\": {\"weight_norm\": 0.06403574087862993, \"weight\": 172}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.17171093267449042, \"weight\": 834}, \"user_id\": {\"weight_norm\": 0.27848101265822783, \"weight\": 748}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.10417953469219683, \"weight\": 506}, \"user_id\": {\"weight_norm\": 0.1746090841399851, \"weight\": 469}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.014618077002264772, \"weight\": 71}, \"user_id\": {\"weight_norm\": 0.02531645569620253, \"weight\": 68}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.019971175622812434, \"weight\": 97}, \"user_id\": {\"weight_norm\": 0.03611317944899479, \"weight\": 97}}, \"type\": \"bad\"}, {\"sourceIndex\": 8, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.007000205888408483, \"weight\": 34}, \"user_id\": {\"weight_norm\": 0.012658227848101266, \"weight\": 34}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.009882643607164917, \"weight\": 48}, \"user_id\": {\"weight_norm\": 0.017870439314966492, \"weight\": 48}}, \"type\": \"bad\"}, {\"sourceIndex\": 8, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.15482808317891703, \"weight\": 752}, \"user_id\": {\"weight_norm\": 0.2542814594192107, \"weight\": 683}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.011529750875025736, \"weight\": 56}, \"user_id\": {\"weight_norm\": 0.020476545048399106, \"weight\": 55}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.01955939880584723, \"weight\": 95}, \"user_id\": {\"weight_norm\": 0.03536857781087118, \"weight\": 95}}, \"type\": \"bad\"}, {\"sourceIndex\": 10, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.07309038501132387, \"weight\": 355}, \"user_id\": {\"weight_norm\": 0.12360387192851824, \"weight\": 332}}, \"type\": \"suit\"}, {\"sourceIndex\": 12, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.8367304920732963, \"weight\": 4064}, \"user_id\": {\"weight_norm\": 0.7501861504095308, \"weight\": 2015}}, \"type\": \"suit\"}, {\"sourceIndex\": 12, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.21453572163887172, \"weight\": 1042}, \"user_id\": {\"weight_norm\": 0.22449739389426657, \"weight\": 603}}, \"type\": \"suit\"}, {\"sourceIndex\": 12, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.10891496808729668, \"weight\": 529}, \"user_id\": {\"weight_norm\": 0.19694713328369323, \"weight\": 529}}, \"type\": \"bad\"}, {\"sourceIndex\": 2, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.5530162651842702, \"weight\": 2686}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 2686}}, \"type\": \"source\"}, {\"sourceIndex\": 2, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.2192711550339716, \"weight\": 1065}, \"user_id\": {\"weight_norm\": 0.3965003723008191, \"weight\": 1065}}, \"type\": \"source\"}, {\"sourceIndex\": 0, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.0041177681696520484, \"weight\": 20}, \"user_id\": {\"weight_norm\": 0.007446016381236039, \"weight\": 20}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.007823759522338892, \"weight\": 38}, \"user_id\": {\"weight_norm\": 0.014147431124348473, \"weight\": 38}}, \"type\": \"bad\"}, {\"sourceIndex\": 0, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.10438542310067943, \"weight\": 507}, \"user_id\": {\"weight_norm\": 0.1768428890543559, \"weight\": 475}}, \"type\": \"nice\"}, {\"sourceIndex\": 9, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.009676755198682314, \"weight\": 47}, \"user_id\": {\"weight_norm\": 0.01675353685778109, \"weight\": 45}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.009470866790199713, \"weight\": 46}, \"user_id\": {\"weight_norm\": 0.01712583767684289, \"weight\": 46}}, \"type\": \"bad\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.021412394482190653, \"weight\": 104}, \"user_id\": {\"weight_norm\": 0.0379746835443038, \"weight\": 102}}, \"type\": \"nice\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.02347127856701668, \"weight\": 114}, \"user_id\": {\"weight_norm\": 0.040208488458674606, \"weight\": 108}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.008647313156269302, \"weight\": 42}, \"user_id\": {\"weight_norm\": 0.01526433358153388, \"weight\": 41}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.019353510397364628, \"weight\": 94}, \"user_id\": {\"weight_norm\": 0.034996276991809384, \"weight\": 94}}, \"type\": \"bad\"}, {\"sourceIndex\": 11, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.11632695079267037, \"weight\": 565}, \"user_id\": {\"weight_norm\": 0.1939687267311988, \"weight\": 521}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.04056001647107268, \"weight\": 197}, \"user_id\": {\"weight_norm\": 0.07073715562174236, \"weight\": 190}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.01955939880584723, \"weight\": 95}, \"user_id\": {\"weight_norm\": 0.034996276991809384, \"weight\": 94}}, \"type\": \"nice\"}, {\"sourceIndex\": 3, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.05538398188182005, \"weight\": 269}, \"user_id\": {\"weight_norm\": 0.09009679821295608, \"weight\": 242}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.08997323450689726, \"weight\": 437}, \"user_id\": {\"weight_norm\": 0.16269545793000745, \"weight\": 437}}, \"type\": \"bad\"}, {\"sourceIndex\": 6, \"targetIndex\": 13, \"weights\": {\"event_id\": {\"weight_norm\": 0.09717932880378835, \"weight\": 472}, \"user_id\": {\"weight_norm\": 0.16046165301563664, \"weight\": 431}}, \"type\": \"nice\"}, {\"sourceIndex\": 6, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.15441630636195183, \"weight\": 750}, \"user_id\": {\"weight_norm\": 0.2308265078183172, \"weight\": 620}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.02470660901791229, \"weight\": 120}, \"user_id\": {\"weight_norm\": 0.04244229337304542, \"weight\": 114}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.03561869466749022, \"weight\": 173}, \"user_id\": {\"weight_norm\": 0.06440804169769174, \"weight\": 173}}, \"type\": \"bad\"}, {\"sourceIndex\": 1, \"targetIndex\": 13, \"weights\": {\"event_id\": {\"weight_norm\": 0.13588634959851761, \"weight\": 660}, \"user_id\": {\"weight_norm\": 0.21667907669396871, \"weight\": 582}}, \"type\": \"nice\"}, {\"sourceIndex\": 1, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.26724315421041794, \"weight\": 1298}, \"user_id\": {\"weight_norm\": 0.347728965003723, \"weight\": 934}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 12, \"weights\": {\"event_id\": {\"weight_norm\": 0.018735845171916822, \"weight\": 91}, \"user_id\": {\"weight_norm\": 0.03276247207743857, \"weight\": 88}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.0253242742433601, \"weight\": 123}, \"user_id\": {\"weight_norm\": 0.04579300074460164, \"weight\": 123}}, \"type\": \"bad\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    linksThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'event_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'event_id',\n",
       "    showWeights: true,\n",
       "    showPercents: false,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: true,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(0),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elMEQHO5_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elMEQHO5')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elMEQHO5_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"el0WPYND\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('el0WPYND')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('el0WPYND')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('el0WPYND')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('el0WPYND').contentDocument\n",
       "      const iframeWindow = document.getElementById('el0WPYND').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elTR4N7H`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '442b958c-1825-4d69-a2b7-a5cd33c64d08',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'el0WPYND_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('el0WPYND')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"el0WPYND_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "stream\\\n",
    "    .add_start_end_events()\\\n",
    "    .transition_graph(\n",
    "        targets={\n",
    "            'positive': ['payment_done', 'cart'],\n",
    "            'negative': 'path_end',\n",
    "            'source': 'path_start'\n",
    "        }\n",
    "    );"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "aUcCs_2tIOCH"
   },
   "source": [
    "### Graph settings\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/",
     "height": 0
    },
    "executionInfo": {
     "elapsed": 19,
     "status": "ok",
     "timestamp": 1683188262200,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "pr3bw0HOJqy-",
    "outputId": "554b685a-d510-47a8-9980-475e66486fb2",
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"elCDVCQE\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elCDVCQE')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elCDVCQE')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elCDVCQE')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elCDVCQE').contentDocument\n",
       "      const iframeWindow = document.getElementById('elCDVCQE').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elSXR3ZW`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '39320cf4-77c3-448f-8171-39f913b46428',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16752, \"source\": 565}, \"user_id\": {\"degree\": 8.32844, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.48822, \"source\": 2172}, \"user_id\": {\"degree\": 15.88037, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 2, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.45888, \"source\": 706}, \"user_id\": {\"degree\": 9.42509, \"source\": 653}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 3, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 34.0, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 4, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.48395, \"source\": 1686}, \"user_id\": {\"degree\": 15.26558, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 5, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.1306, \"source\": 1515}, \"user_id\": {\"degree\": 13.32152, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 6, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72338, \"source\": 834}, \"user_id\": {\"degree\": 10.21435, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 7, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.40708, \"source\": 197}, \"user_id\": {\"degree\": 5.57851, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 8, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.0456, \"source\": 506}, \"user_id\": {\"degree\": 7.89643, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 9, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.28751, \"source\": 1107}, \"user_id\": {\"degree\": 11.95901, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 10, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.64417, \"source\": 5635}, \"user_id\": {\"degree\": 23.81446, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 11, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.87271, \"source\": 2842}, \"user_id\": {\"degree\": 19.98449, \"source\": 1924}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}],\n",
       "    configLinks: [{\"sourceIndex\": 11, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.0004, \"weight\": 0.0004}, \"user_id\": {\"weight_norm\": 0.00059, \"weight\": 0.00059}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.22923, \"weight\": 0.22923}, \"user_id\": {\"weight_norm\": 0.28385, \"weight\": 0.28385}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.67684, \"weight\": 0.67684}, \"user_id\": {\"weight_norm\": 0.80523, \"weight\": 0.80523}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.09354, \"weight\": 0.09354}, \"user_id\": {\"weight_norm\": 0.12114, \"weight\": 0.12114}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.13354, \"weight\": 0.13354}, \"user_id\": {\"weight_norm\": 0.41846, \"weight\": 0.41846}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.37951, \"weight\": 0.37951}, \"user_id\": {\"weight_norm\": 0.63338, \"weight\": 0.63338}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.19886, \"weight\": 0.19886}, \"user_id\": {\"weight_norm\": 0.46776, \"weight\": 0.46776}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.11838, \"weight\": 0.11838}, \"user_id\": {\"weight_norm\": 0.35461, \"weight\": 0.35461}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.16971, \"weight\": 0.16971}, \"user_id\": {\"weight_norm\": 0.45196, \"weight\": 0.45196}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11202, \"weight\": 0.11202}, \"user_id\": {\"weight_norm\": 0.13501, \"weight\": 0.13501}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.52486, \"weight\": 0.52486}, \"user_id\": {\"weight_norm\": 0.58713, \"weight\": 0.58713}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.31844, \"weight\": 0.31844}, \"user_id\": {\"weight_norm\": 0.36813, \"weight\": 0.36813}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04468, \"weight\": 0.04468}, \"user_id\": {\"weight_norm\": 0.05338, \"weight\": 0.05338}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04326, \"weight\": 0.04326}, \"user_id\": {\"weight_norm\": 0.04809, \"weight\": 0.04809}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.95674, \"weight\": 0.95674}, \"user_id\": {\"weight_norm\": 0.96605, \"weight\": 0.96605}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.13625, \"weight\": 0.13625}, \"user_id\": {\"weight_norm\": 0.1455, \"weight\": 0.1455}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.86375, \"weight\": 0.86375}, \"user_id\": {\"weight_norm\": 0.87831, \"weight\": 0.87831}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.79593, \"weight\": 0.79593}, \"user_id\": {\"weight_norm\": 0.94601, \"weight\": 0.94601}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.20407, \"weight\": 0.20407}, \"user_id\": {\"weight_norm\": 0.2831, \"weight\": 0.2831}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.03795, \"weight\": 0.03795}, \"user_id\": {\"weight_norm\": 0.04098, \"weight\": 0.04098}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.96205, \"weight\": 0.96205}, \"user_id\": {\"weight_norm\": 0.97336, \"weight\": 0.97336}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.31126, \"weight\": 0.31126}, \"user_id\": {\"weight_norm\": 0.3125, \"weight\": 0.3125}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.68874, \"weight\": 0.68874}, \"user_id\": {\"weight_norm\": 0.70833, \"weight\": 0.70833}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11254, \"weight\": 0.11254}, \"user_id\": {\"weight_norm\": 0.12343, \"weight\": 0.12343}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04146, \"weight\": 0.04146}, \"user_id\": {\"weight_norm\": 0.04686, \"weight\": 0.04686}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.55775, \"weight\": 0.55775}, \"user_id\": {\"weight_norm\": 0.59543, \"weight\": 0.59543}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.19447, \"weight\": 0.19447}, \"user_id\": {\"weight_norm\": 0.21714, \"weight\": 0.21714}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.09378, \"weight\": 0.09378}, \"user_id\": {\"weight_norm\": 0.10743, \"weight\": 0.10743}}, \"type\": \"suit\"}, {\"sourceIndex\": 2, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.35171, \"weight\": 0.35171}, \"user_id\": {\"weight_norm\": 0.4336, \"weight\": 0.4336}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.55887, \"weight\": 0.55887}, \"user_id\": {\"weight_norm\": 0.62374, \"weight\": 0.62374}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.08942, \"weight\": 0.08942}, \"user_id\": {\"weight_norm\": 0.11469, \"weight\": 0.11469}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.32211, \"weight\": 0.32211}, \"user_id\": {\"weight_norm\": 0.43304, \"weight\": 0.43304}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.63348, \"weight\": 0.63348}, \"user_id\": {\"weight_norm\": 0.69494, \"weight\": 0.69494}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04441, \"weight\": 0.04441}, \"user_id\": {\"weight_norm\": 0.06548, \"weight\": 0.06548}}, \"type\": \"suit\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    linksThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'event_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'event_id',\n",
       "    showWeights: true,\n",
       "    showPercents: true,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: false,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(0),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elCDVCQE_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elCDVCQE')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elCDVCQE_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"el4YS033\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('el4YS033')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('el4YS033')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('el4YS033')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('el4YS033').contentDocument\n",
       "      const iframeWindow = document.getElementById('el4YS033').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elSXR3ZW`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '39320cf4-77c3-448f-8171-39f913b46428',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'el4YS033_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('el4YS033')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"el4YS033_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/plain": [
       "<retentioneering.tooling.transition_graph.transition_graph.TransitionGraph at 0x164a98640>"
      ]
     },
     "execution_count": 13,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "stream.transition_graph(\n",
    "    edges_norm_type='node',\n",
    "    show_weights=True,\n",
    "    show_percents=True,\n",
    "    show_nodes_names=True,\n",
    "    show_all_edges_for_targets=False,\n",
    "    show_nodes_without_links=False\n",
    ")"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import and export graph layout"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"elEFS4KR\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elEFS4KR')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elEFS4KR')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elEFS4KR')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elEFS4KR').contentDocument\n",
       "      const iframeWindow = document.getElementById('elEFS4KR').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elVJA8IH`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: 'd48a165d-946b-42d5-90eb-a86f2af4a5b8',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16751618680259, \"source\": 565}, \"user_id\": {\"degree\": 8.32844087510385, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 609, \"y\": 539}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.488221518115443, \"source\": 2172}, \"user_id\": {\"degree\": 15.880365549709222, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 292, \"y\": 135}, {\"index\": 2, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.458878633420581, \"source\": 706}, \"user_id\": {\"degree\": 9.425090002769316, \"source\": 653}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 371, \"y\": 645}, {\"index\": 3, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 34.0, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 174, \"y\": 480}, {\"index\": 4, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.483950957432153, \"source\": 1686}, \"user_id\": {\"degree\": 15.265577402381613, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 635, \"y\": 162}, {\"index\": 5, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.13059650089544, \"source\": 1515}, \"user_id\": {\"degree\": 13.321517585156467, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 127, \"y\": 165}, {\"index\": 6, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72337787574046, \"source\": 834}, \"user_id\": {\"degree\": 10.214345056770977, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 770, \"y\": 261}, {\"index\": 7, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.407080865132938, \"source\": 197}, \"user_id\": {\"degree\": 5.578510108003323, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 792, \"y\": 545}, {\"index\": 8, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.045598567295771, \"source\": 506}, \"user_id\": {\"degree\": 7.89642758238715, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 556, \"y\": 266}, {\"index\": 9, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.287505166000827, \"source\": 1107}, \"user_id\": {\"degree\": 11.959014123511494, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 707, \"y\": 418}, {\"index\": 10, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.644165863066538, \"source\": 5635}, \"user_id\": {\"degree\": 23.814455829410136, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 465, \"y\": 433}, {\"index\": 11, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.872709739633558, \"source\": 2842}, \"user_id\": {\"degree\": 19.98449183051786, \"source\": 1924}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null, \"x\": 491, \"y\": 187}],\n",
       "    configLinks: [{\"sourceIndex\": 11, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.00020588840848260242, \"weight\": 1}, \"user_id\": {\"weight_norm\": 0.0004962779156327543, \"weight\": 1}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11756228124356599, \"weight\": 571}, \"user_id\": {\"weight_norm\": 0.23722084367245658, \"weight\": 478}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.3471278567016677, \"weight\": 1686}, \"user_id\": {\"weight_norm\": 0.6729528535980149, \"weight\": 1356}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.047971999176446366, \"weight\": 233}, \"user_id\": {\"weight_norm\": 0.10124069478908189, \"weight\": 204}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.35186329009676753, \"weight\": 1709}, \"user_id\": {\"weight_norm\": 0.6570719602977667, \"weight\": 1324}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 4857}, \"user_id\": {\"weight_norm\": 0.9945409429280397, \"weight\": 2004}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.5239859995882232, \"weight\": 2545}, \"user_id\": {\"weight_norm\": 0.7344913151364765, \"weight\": 1480}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.3119209388511427, \"weight\": 1515}, \"user_id\": {\"weight_norm\": 0.5568238213399503, \"weight\": 1122}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.44718962322421246, \"weight\": 2172}, \"user_id\": {\"weight_norm\": 0.7096774193548387, \"weight\": 1430}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.03664813670990323, \"weight\": 178}, \"user_id\": {\"weight_norm\": 0.08535980148883375, \"weight\": 172}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.17171093267449042, \"weight\": 834}, \"user_id\": {\"weight_norm\": 0.37121588089330027, \"weight\": 748}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.10417953469219683, \"weight\": 506}, \"user_id\": {\"weight_norm\": 0.2327543424317618, \"weight\": 469}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.014618077002264772, \"weight\": 71}, \"user_id\": {\"weight_norm\": 0.0337468982630273, \"weight\": 68}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.007000205888408483, \"weight\": 34}, \"user_id\": {\"weight_norm\": 0.01687344913151365, \"weight\": 34}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.15482808317891703, \"weight\": 752}, \"user_id\": {\"weight_norm\": 0.3389578163771712, \"weight\": 683}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.011529750875025736, \"weight\": 56}, \"user_id\": {\"weight_norm\": 0.02729528535980149, \"weight\": 55}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.07309038501132387, \"weight\": 355}, \"user_id\": {\"weight_norm\": 0.16476426799007443, \"weight\": 332}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.8367304920732963, \"weight\": 4064}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 2015}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.21453572163887172, \"weight\": 1042}, \"user_id\": {\"weight_norm\": 0.2992555831265509, \"weight\": 603}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.0041177681696520484, \"weight\": 20}, \"user_id\": {\"weight_norm\": 0.009925558312655087, \"weight\": 20}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.10438542310067943, \"weight\": 507}, \"user_id\": {\"weight_norm\": 0.23573200992555832, \"weight\": 475}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.009676755198682314, \"weight\": 47}, \"user_id\": {\"weight_norm\": 0.022332506203473945, \"weight\": 45}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.021412394482190653, \"weight\": 104}, \"user_id\": {\"weight_norm\": 0.050620347394540945, \"weight\": 102}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.02347127856701668, \"weight\": 114}, \"user_id\": {\"weight_norm\": 0.05359801488833747, \"weight\": 108}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.008647313156269302, \"weight\": 42}, \"user_id\": {\"weight_norm\": 0.020347394540942927, \"weight\": 41}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.11632695079267037, \"weight\": 565}, \"user_id\": {\"weight_norm\": 0.25856079404466503, \"weight\": 521}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.04056001647107268, \"weight\": 197}, \"user_id\": {\"weight_norm\": 0.09429280397022333, \"weight\": 190}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.01955939880584723, \"weight\": 95}, \"user_id\": {\"weight_norm\": 0.04665012406947891, \"weight\": 94}}, \"type\": \"suit\"}, {\"sourceIndex\": 2, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.05538398188182005, \"weight\": 269}, \"user_id\": {\"weight_norm\": 0.12009925558312655, \"weight\": 242}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.09717932880378835, \"weight\": 472}, \"user_id\": {\"weight_norm\": 0.2138957816377171, \"weight\": 431}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.15441630636195183, \"weight\": 750}, \"user_id\": {\"weight_norm\": 0.3076923076923077, \"weight\": 620}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.02470660901791229, \"weight\": 120}, \"user_id\": {\"weight_norm\": 0.05657568238213399, \"weight\": 114}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.13588634959851761, \"weight\": 660}, \"user_id\": {\"weight_norm\": 0.288833746898263, \"weight\": 582}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.26724315421041794, \"weight\": 1298}, \"user_id\": {\"weight_norm\": 0.4635235732009926, \"weight\": 934}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.018735845171916822, \"weight\": 91}, \"user_id\": {\"weight_norm\": 0.04367245657568238, \"weight\": 88}}, \"type\": \"suit\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    linksThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'event_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'event_id',\n",
       "    showWeights: true,\n",
       "    showPercents: false,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: true,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elEFS4KR_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elEFS4KR')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elEFS4KR_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"elGFYKCY\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elGFYKCY')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elGFYKCY')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elGFYKCY')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elGFYKCY').contentDocument\n",
       "      const iframeWindow = document.getElementById('elGFYKCY').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elVJA8IH`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: 'd48a165d-946b-42d5-90eb-a86f2af4a5b8',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elGFYKCY_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elGFYKCY')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elGFYKCY_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/plain": [
       "<retentioneering.tooling.transition_graph.transition_graph.TransitionGraph at 0x164a92ca0>"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "path_link = '/path/to/node_params.json'\n",
    "stream.transition_graph(layout_dump=path_link)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Export the modified eventstream"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 28,
   "metadata": {
    "scrolled": false
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"elH47JHX\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elH47JHX')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elH47JHX')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elH47JHX')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elH47JHX').contentDocument\n",
       "      const iframeWindow = document.getElementById('elH47JHX').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elPROQ1A`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '13babd2d-3a82-4180-8c99-232ad2bc4f1a',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16751618680259, \"source\": 565}, \"user_id\": {\"degree\": 8.32844087510385, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.488221518115443, \"source\": 2172}, \"user_id\": {\"degree\": 15.880365549709222, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 2, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.458878633420581, \"source\": 706}, \"user_id\": {\"degree\": 9.425090002769316, \"source\": 653}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 3, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 34.0, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 4, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.483950957432153, \"source\": 1686}, \"user_id\": {\"degree\": 15.265577402381613, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 5, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.13059650089544, \"source\": 1515}, \"user_id\": {\"degree\": 13.321517585156467, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 6, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72337787574046, \"source\": 834}, \"user_id\": {\"degree\": 10.214345056770977, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 7, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.407080865132938, \"source\": 197}, \"user_id\": {\"degree\": 5.578510108003323, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 8, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.045598567295771, \"source\": 506}, \"user_id\": {\"degree\": 7.89642758238715, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 9, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.287505166000827, \"source\": 1107}, \"user_id\": {\"degree\": 11.959014123511494, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 10, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.644165863066538, \"source\": 5635}, \"user_id\": {\"degree\": 23.814455829410136, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 11, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.872709739633558, \"source\": 2842}, \"user_id\": {\"degree\": 19.98449183051786, \"source\": 1924}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}],\n",
       "    configLinks: [{\"sourceIndex\": 11, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.00020588840848260242, \"weight\": 1}, \"user_id\": {\"weight_norm\": 0.0004962779156327543, \"weight\": 1}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11756228124356599, \"weight\": 571}, \"user_id\": {\"weight_norm\": 0.23722084367245658, \"weight\": 478}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.3471278567016677, \"weight\": 1686}, \"user_id\": {\"weight_norm\": 0.6729528535980149, \"weight\": 1356}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.047971999176446366, \"weight\": 233}, \"user_id\": {\"weight_norm\": 0.10124069478908189, \"weight\": 204}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.35186329009676753, \"weight\": 1709}, \"user_id\": {\"weight_norm\": 0.6570719602977667, \"weight\": 1324}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 4857}, \"user_id\": {\"weight_norm\": 0.9945409429280397, \"weight\": 2004}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.5239859995882232, \"weight\": 2545}, \"user_id\": {\"weight_norm\": 0.7344913151364765, \"weight\": 1480}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.3119209388511427, \"weight\": 1515}, \"user_id\": {\"weight_norm\": 0.5568238213399503, \"weight\": 1122}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.44718962322421246, \"weight\": 2172}, \"user_id\": {\"weight_norm\": 0.7096774193548387, \"weight\": 1430}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.03664813670990323, \"weight\": 178}, \"user_id\": {\"weight_norm\": 0.08535980148883375, \"weight\": 172}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.17171093267449042, \"weight\": 834}, \"user_id\": {\"weight_norm\": 0.37121588089330027, \"weight\": 748}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.10417953469219683, \"weight\": 506}, \"user_id\": {\"weight_norm\": 0.2327543424317618, \"weight\": 469}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.014618077002264772, \"weight\": 71}, \"user_id\": {\"weight_norm\": 0.0337468982630273, \"weight\": 68}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.007000205888408483, \"weight\": 34}, \"user_id\": {\"weight_norm\": 0.01687344913151365, \"weight\": 34}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.15482808317891703, \"weight\": 752}, \"user_id\": {\"weight_norm\": 0.3389578163771712, \"weight\": 683}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.011529750875025736, \"weight\": 56}, \"user_id\": {\"weight_norm\": 0.02729528535980149, \"weight\": 55}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.07309038501132387, \"weight\": 355}, \"user_id\": {\"weight_norm\": 0.16476426799007443, \"weight\": 332}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.8367304920732963, \"weight\": 4064}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 2015}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.21453572163887172, \"weight\": 1042}, \"user_id\": {\"weight_norm\": 0.2992555831265509, \"weight\": 603}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.0041177681696520484, \"weight\": 20}, \"user_id\": {\"weight_norm\": 0.009925558312655087, \"weight\": 20}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.10438542310067943, \"weight\": 507}, \"user_id\": {\"weight_norm\": 0.23573200992555832, \"weight\": 475}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.009676755198682314, \"weight\": 47}, \"user_id\": {\"weight_norm\": 0.022332506203473945, \"weight\": 45}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.021412394482190653, \"weight\": 104}, \"user_id\": {\"weight_norm\": 0.050620347394540945, \"weight\": 102}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.02347127856701668, \"weight\": 114}, \"user_id\": {\"weight_norm\": 0.05359801488833747, \"weight\": 108}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.008647313156269302, \"weight\": 42}, \"user_id\": {\"weight_norm\": 0.020347394540942927, \"weight\": 41}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.11632695079267037, \"weight\": 565}, \"user_id\": {\"weight_norm\": 0.25856079404466503, \"weight\": 521}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.04056001647107268, \"weight\": 197}, \"user_id\": {\"weight_norm\": 0.09429280397022333, \"weight\": 190}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.01955939880584723, \"weight\": 95}, \"user_id\": {\"weight_norm\": 0.04665012406947891, \"weight\": 94}}, \"type\": \"suit\"}, {\"sourceIndex\": 2, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.05538398188182005, \"weight\": 269}, \"user_id\": {\"weight_norm\": 0.12009925558312655, \"weight\": 242}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.09717932880378835, \"weight\": 472}, \"user_id\": {\"weight_norm\": 0.2138957816377171, \"weight\": 431}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.15441630636195183, \"weight\": 750}, \"user_id\": {\"weight_norm\": 0.3076923076923077, \"weight\": 620}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.02470660901791229, \"weight\": 120}, \"user_id\": {\"weight_norm\": 0.05657568238213399, \"weight\": 114}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.13588634959851761, \"weight\": 660}, \"user_id\": {\"weight_norm\": 0.288833746898263, \"weight\": 582}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.26724315421041794, \"weight\": 1298}, \"user_id\": {\"weight_norm\": 0.4635235732009926, \"weight\": 934}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.018735845171916822, \"weight\": 91}, \"user_id\": {\"weight_norm\": 0.04367245657568238, \"weight\": 88}}, \"type\": \"suit\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    linksThreshold: {\"user_id\": 0.0, \"event_id\": 0.0},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'event_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'event_id',\n",
       "    showWeights: true,\n",
       "    showPercents: false,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: true,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(0),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elH47JHX_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elH47JHX')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elH47JHX_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"elGJ3FOL\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elGJ3FOL')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elGJ3FOL')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elGJ3FOL')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elGJ3FOL').contentDocument\n",
       "      const iframeWindow = document.getElementById('elGJ3FOL').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elPROQ1A`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '13babd2d-3a82-4180-8c99-232ad2bc4f1a',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'none',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elGJ3FOL_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elGJ3FOL')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elGJ3FOL_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "tg = stream.transition_graph()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 35,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>event_id</th>\n",
       "      <th>event_type</th>\n",
       "      <th>event_index</th>\n",
       "      <th>event</th>\n",
       "      <th>timestamp</th>\n",
       "      <th>user_id</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>0</th>\n",
       "      <td>723f0b5a-67fc-4a93-8f16-26914841eddb</td>\n",
       "      <td>raw</td>\n",
       "      <td>0</td>\n",
       "      <td>catalog</td>\n",
       "      <td>2019-11-01 17:59:13.273932</td>\n",
       "      <td>219483890</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>1</th>\n",
       "      <td>7dce0d08-55ab-4d0c-b28a-f2381822f25d</td>\n",
       "      <td>raw</td>\n",
       "      <td>1</td>\n",
       "      <td>product1</td>\n",
       "      <td>2019-11-01 17:59:28.459271</td>\n",
       "      <td>219483890</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2</th>\n",
       "      <td>4a56daf4-d0a3-4c58-95e6-dcc712b54384</td>\n",
       "      <td>raw</td>\n",
       "      <td>2</td>\n",
       "      <td>cart</td>\n",
       "      <td>2019-11-01 17:59:29.502214</td>\n",
       "      <td>219483890</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>3</th>\n",
       "      <td>bad3f4ae-8605-40ec-96ef-574d8a9a8269</td>\n",
       "      <td>raw</td>\n",
       "      <td>3</td>\n",
       "      <td>catalog</td>\n",
       "      <td>2019-11-01 17:59:32.557029</td>\n",
       "      <td>219483890</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>4</th>\n",
       "      <td>ad44e840-080b-4c70-91d5-beebbbc08bc5</td>\n",
       "      <td>raw</td>\n",
       "      <td>4</td>\n",
       "      <td>catalog</td>\n",
       "      <td>2019-11-01 21:38:19.283663</td>\n",
       "      <td>964964743</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>...</th>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>32278</th>\n",
       "      <td>2e2f2db0-2ee1-43aa-bc3b-5fe16d231196</td>\n",
       "      <td>raw</td>\n",
       "      <td>32278</td>\n",
       "      <td>main</td>\n",
       "      <td>2020-04-29 12:47:39.956925</td>\n",
       "      <td>501098384</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>32279</th>\n",
       "      <td>3715634c-2854-4f52-8783-3708e943505c</td>\n",
       "      <td>raw</td>\n",
       "      <td>32279</td>\n",
       "      <td>catalog</td>\n",
       "      <td>2020-04-29 12:47:40.975732</td>\n",
       "      <td>501098384</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>32280</th>\n",
       "      <td>ec1f00ac-8530-42db-a398-0fa0834f8f27</td>\n",
       "      <td>raw</td>\n",
       "      <td>32280</td>\n",
       "      <td>catalog</td>\n",
       "      <td>2020-04-29 12:48:01.809577</td>\n",
       "      <td>501098384</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>32281</th>\n",
       "      <td>d8bb9a7e-013b-4418-99d1-bc3f81ef8d22</td>\n",
       "      <td>raw</td>\n",
       "      <td>32281</td>\n",
       "      <td>main</td>\n",
       "      <td>2020-04-29 12:48:01.938488</td>\n",
       "      <td>501098384</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>32282</th>\n",
       "      <td>6124f321-41dc-4202-9e9c-0f92006f6a48</td>\n",
       "      <td>raw</td>\n",
       "      <td>32282</td>\n",
       "      <td>catalog</td>\n",
       "      <td>2020-04-29 12:48:06.595390</td>\n",
       "      <td>501098384</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "<p>32283 rows × 6 columns</p>\n",
       "</div>"
      ],
      "text/plain": [
       "                                   event_id event_type  event_index     event  \\\n",
       "0      723f0b5a-67fc-4a93-8f16-26914841eddb        raw            0   catalog   \n",
       "1      7dce0d08-55ab-4d0c-b28a-f2381822f25d        raw            1  product1   \n",
       "2      4a56daf4-d0a3-4c58-95e6-dcc712b54384        raw            2      cart   \n",
       "3      bad3f4ae-8605-40ec-96ef-574d8a9a8269        raw            3   catalog   \n",
       "4      ad44e840-080b-4c70-91d5-beebbbc08bc5        raw            4   catalog   \n",
       "...                                     ...        ...          ...       ...   \n",
       "32278  2e2f2db0-2ee1-43aa-bc3b-5fe16d231196        raw        32278      main   \n",
       "32279  3715634c-2854-4f52-8783-3708e943505c        raw        32279   catalog   \n",
       "32280  ec1f00ac-8530-42db-a398-0fa0834f8f27        raw        32280   catalog   \n",
       "32281  d8bb9a7e-013b-4418-99d1-bc3f81ef8d22        raw        32281      main   \n",
       "32282  6124f321-41dc-4202-9e9c-0f92006f6a48        raw        32282   catalog   \n",
       "\n",
       "                       timestamp    user_id  \n",
       "0     2019-11-01 17:59:13.273932  219483890  \n",
       "1     2019-11-01 17:59:28.459271  219483890  \n",
       "2     2019-11-01 17:59:29.502214  219483890  \n",
       "3     2019-11-01 17:59:32.557029  219483890  \n",
       "4     2019-11-01 21:38:19.283663  964964743  \n",
       "...                          ...        ...  \n",
       "32278 2020-04-29 12:47:39.956925  501098384  \n",
       "32279 2020-04-29 12:47:40.975732  501098384  \n",
       "32280 2020-04-29 12:48:01.809577  501098384  \n",
       "32281 2020-04-29 12:48:01.938488  501098384  \n",
       "32282 2020-04-29 12:48:06.595390  501098384  \n",
       "\n",
       "[32283 rows x 6 columns]"
      ]
     },
     "execution_count": 35,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# group some nodes in GUI and perform recalculation before running this cell\n",
    "tg.recalculation_result.to_dataframe()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "6FFm6C3TbOZW"
   },
   "source": [
    "## Transition matrix"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 31,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/",
     "height": 0
    },
    "executionInfo": {
     "elapsed": 19,
     "status": "ok",
     "timestamp": 1683188262201,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "0kOE5qD3cZ6H",
    "outputId": "cafa5be9-303e-4cd5-d310-9c6cf45bf9be"
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>cart</th>\n",
       "      <th>catalog</th>\n",
       "      <th>delivery_choice</th>\n",
       "      <th>main</th>\n",
       "      <th>product1</th>\n",
       "      <th>product2</th>\n",
       "      <th>delivery_courier</th>\n",
       "      <th>delivery_pickup</th>\n",
       "      <th>payment_choice</th>\n",
       "      <th>payment_card</th>\n",
       "      <th>payment_done</th>\n",
       "      <th>payment_cash</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>cart</th>\n",
       "      <td>0.000594</td>\n",
       "      <td>0.283848</td>\n",
       "      <td>0.805226</td>\n",
       "      <td>0.121140</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>catalog</th>\n",
       "      <td>0.418458</td>\n",
       "      <td>0.633375</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.467762</td>\n",
       "      <td>0.354614</td>\n",
       "      <td>0.45196</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>delivery_choice</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.135008</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.053375</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.587127</td>\n",
       "      <td>0.368132</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>main</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.946009</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.283099</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>product1</th>\n",
       "      <td>0.433602</td>\n",
       "      <td>0.623742</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.114688</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>product2</th>\n",
       "      <td>0.433036</td>\n",
       "      <td>0.694940</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.065476</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>delivery_courier</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.048091</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.966054</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>delivery_pickup</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.145503</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.878307</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>payment_choice</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.123429</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.046857</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.595429</td>\n",
       "      <td>0.107429</td>\n",
       "      <td>0.217143</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>payment_card</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.040984</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.973361</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>payment_done</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>1.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>payment_cash</th>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.312500</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.00000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.000000</td>\n",
       "      <td>0.708333</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                      cart   catalog  delivery_choice      main  product1  \\\n",
       "cart              0.000594  0.283848         0.805226  0.121140  0.000000   \n",
       "catalog           0.418458  0.633375         0.000000  0.467762  0.354614   \n",
       "delivery_choice   0.000000  0.135008         0.000000  0.053375  0.000000   \n",
       "main              0.000000  0.946009         0.000000  0.283099  0.000000   \n",
       "product1          0.433602  0.623742         0.000000  0.114688  0.000000   \n",
       "product2          0.433036  0.694940         0.000000  0.065476  0.000000   \n",
       "delivery_courier  0.000000  0.000000         0.000000  0.048091  0.000000   \n",
       "delivery_pickup   0.000000  0.000000         0.000000  0.145503  0.000000   \n",
       "payment_choice    0.000000  0.123429         0.000000  0.046857  0.000000   \n",
       "payment_card      0.000000  0.000000         0.000000  0.040984  0.000000   \n",
       "payment_done      0.000000  0.000000         0.000000  1.000000  0.000000   \n",
       "payment_cash      0.000000  0.000000         0.000000  0.312500  0.000000   \n",
       "\n",
       "                  product2  delivery_courier  delivery_pickup  payment_choice  \\\n",
       "cart               0.00000          0.000000         0.000000        0.000000   \n",
       "catalog            0.45196          0.000000         0.000000        0.000000   \n",
       "delivery_choice    0.00000          0.587127         0.368132        0.000000   \n",
       "main               0.00000          0.000000         0.000000        0.000000   \n",
       "product1           0.00000          0.000000         0.000000        0.000000   \n",
       "product2           0.00000          0.000000         0.000000        0.000000   \n",
       "delivery_courier   0.00000          0.000000         0.000000        0.966054   \n",
       "delivery_pickup    0.00000          0.000000         0.000000        0.878307   \n",
       "payment_choice     0.00000          0.000000         0.000000        0.000000   \n",
       "payment_card       0.00000          0.000000         0.000000        0.000000   \n",
       "payment_done       0.00000          0.000000         0.000000        0.000000   \n",
       "payment_cash       0.00000          0.000000         0.000000        0.000000   \n",
       "\n",
       "                  payment_card  payment_done  payment_cash  \n",
       "cart                  0.000000      0.000000      0.000000  \n",
       "catalog               0.000000      0.000000      0.000000  \n",
       "delivery_choice       0.000000      0.000000      0.000000  \n",
       "main                  0.000000      0.000000      0.000000  \n",
       "product1              0.000000      0.000000      0.000000  \n",
       "product2              0.000000      0.000000      0.000000  \n",
       "delivery_courier      0.000000      0.000000      0.000000  \n",
       "delivery_pickup       0.000000      0.000000      0.000000  \n",
       "payment_choice        0.595429      0.107429      0.217143  \n",
       "payment_card          0.000000      0.973361      0.000000  \n",
       "payment_done          0.000000      0.000000      0.000000  \n",
       "payment_cash          0.000000      0.708333      0.000000  "
      ]
     },
     "execution_count": 31,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "stream.transition_matrix(norm_type='node', weight_col='user_id')"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "id": "yPm321DlbOjv"
   },
   "source": [
    "## Using a separate instance"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 32,
   "metadata": {
    "colab": {
     "base_uri": "https://localhost:8080/",
     "height": 0
    },
    "executionInfo": {
     "elapsed": 427,
     "status": "ok",
     "timestamp": 1683188262610,
     "user": {
      "displayName": "Julia Ostanina",
      "userId": "13687663492290466770"
     },
     "user_tz": -120
    },
    "id": "FXOVZ8J-dEjR",
    "outputId": "11468aea-f86f-4b8c-9aa2-ff44852621b8"
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<iframe id=\"elO5G9B3\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elO5G9B3')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elO5G9B3')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elO5G9B3')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elO5G9B3').contentDocument\n",
       "      const iframeWindow = document.getElementById('elO5G9B3').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elP2DHW5`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '413cb382-47d1-4a88-b87a-a7a898a8b55d',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: [{\"index\": 0, \"name\": \"payment_card\", \"degree\": {\"event_id\": {\"degree\": 5.16752, \"source\": 565}, \"user_id\": {\"degree\": 8.32844, \"source\": 521}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 1, \"name\": \"product2\", \"degree\": {\"event_id\": {\"degree\": 8.48822, \"source\": 2172}, \"user_id\": {\"degree\": 15.88037, \"source\": 1430}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 2, \"name\": \"payment_done\", \"degree\": {\"event_id\": {\"degree\": 5.45888, \"source\": 706}, \"user_id\": {\"degree\": 9.42509, \"source\": 653}}, \"type\": \"nice_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 3, \"name\": \"catalog\", \"degree\": {\"event_id\": {\"degree\": 34.0, \"source\": 14518}, \"user_id\": {\"degree\": 34.0, \"source\": 3611}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 4, \"name\": \"delivery_choice\", \"degree\": {\"event_id\": {\"degree\": 7.48395, \"source\": 1686}, \"user_id\": {\"degree\": 15.26558, \"source\": 1356}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 5, \"name\": \"product1\", \"degree\": {\"event_id\": {\"degree\": 7.1306, \"source\": 1515}, \"user_id\": {\"degree\": 13.32152, \"source\": 1122}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 6, \"name\": \"delivery_courier\", \"degree\": {\"event_id\": {\"degree\": 5.72338, \"source\": 834}, \"user_id\": {\"degree\": 10.21435, \"source\": 748}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 7, \"name\": \"payment_cash\", \"degree\": {\"event_id\": {\"degree\": 4.40708, \"source\": 197}, \"user_id\": {\"degree\": 5.57851, \"source\": 190}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 8, \"name\": \"delivery_pickup\", \"degree\": {\"event_id\": {\"degree\": 5.0456, \"source\": 506}, \"user_id\": {\"degree\": 7.89643, \"source\": 469}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 9, \"name\": \"payment_choice\", \"degree\": {\"event_id\": {\"degree\": 6.28751, \"source\": 1107}, \"user_id\": {\"degree\": 11.95901, \"source\": 958}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 10, \"name\": \"main\", \"degree\": {\"event_id\": {\"degree\": 15.64417, \"source\": 5635}, \"user_id\": {\"degree\": 23.81446, \"source\": 2385}}, \"type\": \"suit_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}, {\"index\": 11, \"name\": \"cart\", \"degree\": {\"event_id\": {\"degree\": 9.87271, \"source\": 2842}, \"user_id\": {\"degree\": 19.98449, \"source\": 1924}}, \"type\": \"nice_node\", \"active\": true, \"alias\": false, \"parent\": null, \"changed_name\": null}],\n",
       "    configLinks: [{\"sourceIndex\": 11, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.0004, \"weight\": 0.0004}, \"user_id\": {\"weight_norm\": 0.00059, \"weight\": 0.00059}}, \"type\": \"nice\"}, {\"sourceIndex\": 11, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.22923, \"weight\": 0.22923}, \"user_id\": {\"weight_norm\": 0.28385, \"weight\": 0.28385}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 4, \"weights\": {\"event_id\": {\"weight_norm\": 0.67684, \"weight\": 0.67684}, \"user_id\": {\"weight_norm\": 0.80523, \"weight\": 0.80523}}, \"type\": \"suit\"}, {\"sourceIndex\": 11, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.09354, \"weight\": 0.09354}, \"user_id\": {\"weight_norm\": 0.12114, \"weight\": 0.12114}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.13354, \"weight\": 0.13354}, \"user_id\": {\"weight_norm\": 0.41846, \"weight\": 0.41846}}, \"type\": \"nice\"}, {\"sourceIndex\": 3, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.37951, \"weight\": 0.37951}, \"user_id\": {\"weight_norm\": 0.63338, \"weight\": 0.63338}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.19886, \"weight\": 0.19886}, \"user_id\": {\"weight_norm\": 0.46776, \"weight\": 0.46776}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 5, \"weights\": {\"event_id\": {\"weight_norm\": 0.11838, \"weight\": 0.11838}, \"user_id\": {\"weight_norm\": 0.35461, \"weight\": 0.35461}}, \"type\": \"suit\"}, {\"sourceIndex\": 3, \"targetIndex\": 1, \"weights\": {\"event_id\": {\"weight_norm\": 0.16971, \"weight\": 0.16971}, \"user_id\": {\"weight_norm\": 0.45196, \"weight\": 0.45196}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11202, \"weight\": 0.11202}, \"user_id\": {\"weight_norm\": 0.13501, \"weight\": 0.13501}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 6, \"weights\": {\"event_id\": {\"weight_norm\": 0.52486, \"weight\": 0.52486}, \"user_id\": {\"weight_norm\": 0.58713, \"weight\": 0.58713}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 8, \"weights\": {\"event_id\": {\"weight_norm\": 0.31844, \"weight\": 0.31844}, \"user_id\": {\"weight_norm\": 0.36813, \"weight\": 0.36813}}, \"type\": \"suit\"}, {\"sourceIndex\": 4, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04468, \"weight\": 0.04468}, \"user_id\": {\"weight_norm\": 0.05338, \"weight\": 0.05338}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04326, \"weight\": 0.04326}, \"user_id\": {\"weight_norm\": 0.04809, \"weight\": 0.04809}}, \"type\": \"suit\"}, {\"sourceIndex\": 6, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.95674, \"weight\": 0.95674}, \"user_id\": {\"weight_norm\": 0.96605, \"weight\": 0.96605}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.13625, \"weight\": 0.13625}, \"user_id\": {\"weight_norm\": 0.1455, \"weight\": 0.1455}}, \"type\": \"suit\"}, {\"sourceIndex\": 8, \"targetIndex\": 9, \"weights\": {\"event_id\": {\"weight_norm\": 0.86375, \"weight\": 0.86375}, \"user_id\": {\"weight_norm\": 0.87831, \"weight\": 0.87831}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.79593, \"weight\": 0.79593}, \"user_id\": {\"weight_norm\": 0.94601, \"weight\": 0.94601}}, \"type\": \"suit\"}, {\"sourceIndex\": 10, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.20407, \"weight\": 0.20407}, \"user_id\": {\"weight_norm\": 0.2831, \"weight\": 0.2831}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.03795, \"weight\": 0.03795}, \"user_id\": {\"weight_norm\": 0.04098, \"weight\": 0.04098}}, \"type\": \"suit\"}, {\"sourceIndex\": 0, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.96205, \"weight\": 0.96205}, \"user_id\": {\"weight_norm\": 0.97336, \"weight\": 0.97336}}, \"type\": \"nice\"}, {\"sourceIndex\": 7, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.31126, \"weight\": 0.31126}, \"user_id\": {\"weight_norm\": 0.3125, \"weight\": 0.3125}}, \"type\": \"suit\"}, {\"sourceIndex\": 7, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.68874, \"weight\": 0.68874}, \"user_id\": {\"weight_norm\": 0.70833, \"weight\": 0.70833}}, \"type\": \"nice\"}, {\"sourceIndex\": 9, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.11254, \"weight\": 0.11254}, \"user_id\": {\"weight_norm\": 0.12343, \"weight\": 0.12343}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04146, \"weight\": 0.04146}, \"user_id\": {\"weight_norm\": 0.04686, \"weight\": 0.04686}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 0, \"weights\": {\"event_id\": {\"weight_norm\": 0.55775, \"weight\": 0.55775}, \"user_id\": {\"weight_norm\": 0.59543, \"weight\": 0.59543}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 7, \"weights\": {\"event_id\": {\"weight_norm\": 0.19447, \"weight\": 0.19447}, \"user_id\": {\"weight_norm\": 0.21714, \"weight\": 0.21714}}, \"type\": \"suit\"}, {\"sourceIndex\": 9, \"targetIndex\": 2, \"weights\": {\"event_id\": {\"weight_norm\": 0.09378, \"weight\": 0.09378}, \"user_id\": {\"weight_norm\": 0.10743, \"weight\": 0.10743}}, \"type\": \"nice\"}, {\"sourceIndex\": 2, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}, \"user_id\": {\"weight_norm\": 1.0, \"weight\": 1.0}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.35171, \"weight\": 0.35171}, \"user_id\": {\"weight_norm\": 0.4336, \"weight\": 0.4336}}, \"type\": \"nice\"}, {\"sourceIndex\": 5, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.55887, \"weight\": 0.55887}, \"user_id\": {\"weight_norm\": 0.62374, \"weight\": 0.62374}}, \"type\": \"suit\"}, {\"sourceIndex\": 5, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.08942, \"weight\": 0.08942}, \"user_id\": {\"weight_norm\": 0.11469, \"weight\": 0.11469}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 11, \"weights\": {\"event_id\": {\"weight_norm\": 0.32211, \"weight\": 0.32211}, \"user_id\": {\"weight_norm\": 0.43304, \"weight\": 0.43304}}, \"type\": \"nice\"}, {\"sourceIndex\": 1, \"targetIndex\": 3, \"weights\": {\"event_id\": {\"weight_norm\": 0.63348, \"weight\": 0.63348}, \"user_id\": {\"weight_norm\": 0.69494, \"weight\": 0.69494}}, \"type\": \"suit\"}, {\"sourceIndex\": 1, \"targetIndex\": 10, \"weights\": {\"event_id\": {\"weight_norm\": 0.04441, \"weight\": 0.04441}, \"user_id\": {\"weight_norm\": 0.06548, \"weight\": 0.06548}}, \"type\": \"suit\"}],\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: {\"event_id\": 500},\n",
       "    linksThreshold: {\"user_id\": 0.12},\n",
       "    shownNodesCol: 'event_id',\n",
       "    shownLinksWeight: 'user_id',\n",
       "    selectedNodesColForThresholds: 'event_id',\n",
       "    selectedLinksWeightForThresholds: 'user_id',\n",
       "    showWeights: true,\n",
       "    showPercents: false,\n",
       "    showNodesNames: true,\n",
       "    showAllEdgesForTargets: true,\n",
       "    showNodesWithoutLinks: false,\n",
       "    showEdgesInfoOnHover: true,\n",
       "    useLayoutDump: Boolean(0),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elO5G9B3_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elO5G9B3')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elO5G9B3_template\">\n",
       "  <!DOCTYPE html>\n",
       "<html lang=\"en\">\n",
       "<head>\n",
       "    <meta charset=\"UTF-8\"/>\n",
       "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no\"/>\n",
       "    <meta name=\"theme-color\" content=\"#000000\"/>\n",
       "    <link rel=\"preconnect\" href=\"https://fonts.googleapis.com\"/>\n",
       "    <link crossorigin rel=\"preconnect\" href=\"https://fonts.gstatic.com\"/>\n",
       "    <link\n",
       "            crossorigin\n",
       "            href=\"https://fonts.googleapis.com/css2?family=Open+Sans:wght@400;600&display=swap\"\n",
       "            rel=\"stylesheet\"\n",
       "    />\n",
       "    <title>Retentioneering Transition Graph Application</title>\n",
       "</head>\n",
       "\n",
       "<body>\n",
       "<noscript>You need to enable JavaScript to run this app.</noscript>\n",
       "<iframe id=\"elETL9FE\" src=\"about:blank\" width=\"960\" height=\"600\"\n",
       "  onload=\"window.reteLoadedIframes = window.reteLoadedIframes || []; window.reteLoadedIframes.push('elETL9FE')\">\n",
       "</iframe>\n",
       "<script>\n",
       "  // We render our widgets inside of iframes (to isolate javascript).\n",
       "  // The @jupyterlab/services library we use reads the basic URL parameters\n",
       "  // for the Jupyter Kernel Gateway and ws connections) from the DOM.\n",
       "  // These parameters are not present in the nested iframe.\n",
       "  // This function copies this configuration to the DOM of the current\n",
       "  // iframe so that PageConfig (https://github.com/jupyterlab/jupyterlab/blob/master/packages/coreutils/src/pageconfig.ts#L36) can retrieve them\n",
       "  function proxyNotebookEnv({ win, doc }) {\n",
       "    const topWindow = window.top\n",
       "\n",
       "    // For classic notebook\n",
       "    try {\n",
       "      const topBody = topWindow.document.body\n",
       "\n",
       "      for (const [name, val] of Object.entries(topBody.dataset)) {\n",
       "        doc.body.dataset[name] = val\n",
       "      }\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "\n",
       "    // Ror Jupyter lab\n",
       "    try {\n",
       "      if (doc.getElementById('jupyter-config-data')) {\n",
       "        return\n",
       "      }\n",
       "      const el = topWindow.document.getElementById('jupyter-config-data')\n",
       "\n",
       "      if (!el) {\n",
       "        return\n",
       "      }\n",
       "\n",
       "      const configData = el.textContent\n",
       "      const id = el.getAttribute('id')\n",
       "      const type = el.getAttribute('type')\n",
       "\n",
       "      const config = doc.createElement('script')\n",
       "      if (id) {\n",
       "        config.setAttribute('id', id)\n",
       "      }\n",
       "      if (type) {\n",
       "        config.setAttribute('type', type)\n",
       "      }\n",
       "      config.textContent = configData\n",
       "      doc.body.appendChild(config)\n",
       "    } catch (error) {\n",
       "      console.error(error)\n",
       "    }\n",
       "  }\n",
       "\n",
       "  function createDoctype({ doc }) {\n",
       "    const newDoctype = doc.implementation.createDocumentType('html', '', '')\n",
       "\n",
       "    if (!doc.doctype) {\n",
       "      doc.insertBefore(newDoctype, doc.childNodes[0])\n",
       "    }\n",
       "  }\n",
       "</script><script>\n",
       "  (function () {\n",
       "    console.info('Initialize graph IFRAME')\n",
       "\n",
       "    const iframe = document.getElementById('elETL9FE')\n",
       "    const iframeLoaded = window.reteLoadedIframes && window.reteLoadedIframes.includes('elETL9FE')\n",
       "\n",
       "    const init = () => {\n",
       "      console.info('Graph IFRAME loaded')\n",
       "\n",
       "      const iframeDocument = document.getElementById('elETL9FE').contentDocument\n",
       "      const iframeWindow = document.getElementById('elETL9FE').contentDocument\n",
       "\n",
       "      iframeDocument.body.innerHTML = '<div id=\"root\"></div>'\n",
       "\n",
       "      createDoctype({ doc: iframeDocument })\n",
       "      proxyNotebookEnv({ win: iframeWindow, doc: iframeDocument })\n",
       "\n",
       "      const graphScript = iframeDocument.createElement('script')\n",
       "\n",
       "      graphScript.src = `https://static.server.retentioneering.com/package/@rete/transition-graph/version/1/dist/transition-graph.umd.js?id=elP2DHW5`\n",
       "\n",
       "      graphScript.addEventListener('load', () => {\n",
       "        console.info('JavaScript library loaded')\n",
       "\n",
       "        const initGraph = iframeDocument.createElement('script')\n",
       "\n",
       "        initGraph.innerHTML = `\n",
       "    initialize({\n",
       "    serverId: '413cb382-47d1-4a88-b87a-a7a898a8b55d',\n",
       "    env: 'classic',\n",
       "    kernelId: \"9554465c-7c0e-4347-85ec-1b025290c6ce\",\n",
       "    selectedNormalizationTypeId: 'node',\n",
       "    configNodes: <%= nodes %>,\n",
       "    configLinks: <%= links %>,\n",
       "    nodesColsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    linksWeightsNames: ['event_id', 'event_id', 'user_id'],\n",
       "    nodesThreshold: <%= nodes_threshold %>,\n",
       "    linksThreshold: <%= links_threshold %>,\n",
       "    shownNodesCol: '<%= shown_nodes_col %>',\n",
       "    shownLinksWeight: '<%= shown_links_weight %>',\n",
       "    selectedNodesColForThresholds: '<%= selected_nodes_col_for_thresholds %>',\n",
       "    selectedLinksWeightForThresholds: '<%= selected_links_weight_for_thresholds %>',\n",
       "    showWeights: <%= show_weights %>,\n",
       "    showPercents: <%= show_percents %>,\n",
       "    showNodesNames: <%= show_nodes_names %>,\n",
       "    showAllEdgesForTargets: <%= show_all_edges_for_targets %>,\n",
       "    showNodesWithoutLinks: <%= show_nodes_without_links %>,\n",
       "    showEdgesInfoOnHover: <%= show_edges_info_on_hover %>,\n",
       "    useLayoutDump: Boolean(1),\n",
       "    weightTemplate: undefined,\n",
       "    trackingHWID: 'fdaa9a9e-9fe1-4593-b94c-0099fabe092e'\n",
       "})\n",
       "`\n",
       "        iframeDocument.body.appendChild(initGraph)\n",
       "      })\n",
       "\n",
       "      iframeDocument.head.appendChild(graphScript)\n",
       "      iframeDocument.body.dataset.templateId = 'elETL9FE_template'\n",
       "\n",
       "      console.info('Graph IFRAME initialized')\n",
       "    }\n",
       "\n",
       "    console.log(`Graph IFRAME loaded: ` + iframeLoaded)\n",
       "\n",
       "    if (iframeLoaded) {\n",
       "      init()\n",
       "      return\n",
       "    }\n",
       "\n",
       "    iframe.onload = () => {\n",
       "      init()\n",
       "      window.reteLoadedIframes = window.reteLoadedIframes || []\n",
       "      window.reteLoadedIframes.push('elETL9FE')\n",
       "    }\n",
       "  })()\n",
       "</script>\n",
       "\n",
       "<template id=\"elETL9FE_template\">\n",
       "  \n",
       "</template>\n",
       "</body>\n",
       "</html>\n",
       "</template>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "from retentioneering.tooling.transition_graph import TransitionGraph\n",
    "\n",
    "tg = TransitionGraph(stream)\n",
    "    \n",
    "tg.plot(edges_norm_type='node',\n",
    "    edges_weight_col='user_id',\n",
    "    edges_threshold={'user_id': 0.12},\n",
    "    nodes_threshold={'event_id': 500},\n",
    "    targets={'positive': ['payment_done', 'cart']}\n",
    ")"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "colab": {
   "provenance": [
    {
     "file_id": "1NkFbLAdIQ_3XqHnSObM_WtDOmlu6ig_Q",
     "timestamp": 1671020860970
    },
    {
     "file_id": "1DBNz_5rl_xErD-g5tZYZP8CMniLOI9Zn",
     "timestamp": 1670969297847
    }
   ],
   "toc_visible": true
  },
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.2"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 1
}
